@@ -64,6 +64,9 @@ namespace Banter
 		protected ConnectionInfo connInfo;
 		protected IConnection tlpConnection;
 		protected IConnectionManager connManager;
+		protected IChannelGroup localInvitationGroup;
+		protected IChannelGroup remoteInvitationGroup;
+		protected IChannelGroup subscribedGroup;
 
 		// Connected Handlers
 		protected bool aliasConnected = false;
@@ -179,6 +182,52 @@ namespace Banter
 		{
 			get {return tlpConnection;}
 		}
+
+		public string[] ReceivedInvitations
+		{
+			get
+			{
+				try {
+					return 
+						tlpConnection.InspectHandles (
+							HandleType.Contact,
+							localInvitationGroup.LocalPendingMembers);
+				} catch{}
+				return null;
+			}
+		}
+		
+		public uint[] ReceivedInvitationHandles
+		{
+			get
+			{
+				try {return localInvitationGroup.LocalPendingMembers;} catch{}
+				return null;
+			}
+		}
+		
+		public string[] SentInvitations
+		{
+			get
+			{
+				try {
+					return 
+						tlpConnection.InspectHandles (
+							HandleType.Contact,
+							remoteInvitationGroup.RemotePendingMembers);
+				} catch{}
+				return null;
+			}
+		}
+		
+		public uint[] SentInvitationHandles
+		{
+			get
+			{
+				try {return remoteInvitationGroup.RemotePendingMembers;} catch{}
+				return null;
+			}
+		}
 		
 		#region Constructors
 		public Account ()
@@ -206,6 +255,7 @@ namespace Banter
 		/// </summary>
 		protected void SetupSupportedInterfaces ()
 		{
+			Logger.Debug ("SetupSupportedInterfaces - called");
 			foreach (string iname in tlpConnection.Interfaces)
 			{
 				switch (iname)
@@ -238,6 +288,54 @@ namespace Banter
 			}
 		}
 
+		/// <summary>
+		/// Method to open and maintain outstanding channels
+		/// to the publish and subscribe lists
+		/// </summary>
+		protected void SetupGroupChannels ()
+		{
+			Logger.Debug ("SetupGroupChannels - called");
+			
+			string[] pubArgs = {"publish"};
+			string[] subArgs = {"subscribe"};
+			ObjectPath objectPath;
+			
+			try {
+				// First setup a channel to the publish list
+				uint[] pubHandles = tlpConnection.RequestHandles (HandleType.List, pubArgs);
+				objectPath = 
+					tlpConnection.RequestChannel (
+						org.freedesktop.Telepathy.ChannelType.ContactList, 
+						HandleType.List, 
+						pubHandles[0], 
+						true);
+						
+				localInvitationGroup = 
+					Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, objectPath);
+				localInvitationGroup.MembersChanged += OnPublishedMembersChanged;
+				
+				// Next setup a channel to the subscribe list
+				uint[] subHandles = tlpConnection.RequestHandles (HandleType.List, subArgs);
+				objectPath = 
+					tlpConnection.RequestChannel (
+						org.freedesktop.Telepathy.ChannelType.ContactList, 
+						HandleType.List, 
+						subHandles[0], 
+						true);
+						
+				remoteInvitationGroup = 
+					Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, objectPath);
+					
+				subscribedGroup = remoteInvitationGroup;
+				subscribedGroup.MembersChanged += OnSubscribedMembersChanged;
+				
+			} catch (Exception sgc) {
+				Logger.Debug ("Failed setting up group channels");
+				Logger.Debug (sgc.Message);
+				Logger.Debug (sgc.StackTrace);
+			}
+		}
+
 		/// <summary>
 		/// Telepathy callback when state is changing on the connection
 		/// </summary>
@@ -259,6 +357,7 @@ namespace Banter
 					try
 					{
 						SetupSupportedInterfaces ();
+						SetupGroupChannels ();
 						SetupMe ();
 						SetupProviderUsers ();
 						AdvertiseCapabilities ();
@@ -350,27 +449,100 @@ namespace Banter
 				Logger.Debug (sm.StackTrace);
 			}
 		}
-		
+
 		protected void SetupProviderUsers ()
 		{
-			//string[] args = {"subscribe"};
-			string[] args = {"known"};
-			uint[] memberHandles = tlpConnection.RequestHandles (HandleType.List, args);
-			ObjectPath op = 
-				tlpConnection.RequestChannel (
-					org.freedesktop.Telepathy.ChannelType.ContactList, 
-					HandleType.List, 
-					memberHandles[0], 
-					true);
+			Logger.Debug ("SetupProviderUsers - called");
+			ProviderUser providerUser;
+			string key;
+			
+			// First add any sent invitation users
+			Logger.Debug ("Adding SentInvitation users");
+			uint[] invitedHandles = this.SentInvitationHandles;
+			if (invitedHandles != null && invitedHandles.Length > 0) {
+				string[] invitedUsers = this.SentInvitations;
+				
+				for (int i = 0; i < invitedUsers.Length; i++) {
 					
-			//Logger.Debug ("# known contacts: {0}", memberHandles.Length);
+					Logger.Debug ("  {0}", invitedUsers[i]);
 					
-			IChannelGroup cl = Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, op);
-			string[] members = tlpConnection.InspectHandles (HandleType.Contact, cl.Members);
+					// update the provider user objects
+					key = ProviderUserManager.CreateKey (invitedUsers[i], protocol);
+					providerUser = null;
+					try {
+						providerUser = ProviderUserManager.GetProviderUser (key);
+						providerUser.TlpConnection = tlpConnection;
+						providerUser.AccountName = this.Name;
+						providerUser.Protocol = this.Protocol;
+						providerUser.ID = invitedHandles[i];
+						providerUser.Relationship = ProviderUserRelationship.SentInvitation;
+					} 
+					catch (Exception fff) {
+						Logger.Debug ("Failed to get ProviderUser {0}", key);
+						Logger.Debug (fff.Message);
+					}
+						
+					if (providerUser == null) {
+						try {
+							providerUser =
+								ProviderUserManager.CreateProviderUser (
+									invitedUsers[i], 
+									this.protocol, 
+									ProviderUserRelationship.SentInvitation);
+							providerUser.TlpConnection = tlpConnection;
+							providerUser.AccountName = this.Name;
+							providerUser.ID = invitedHandles[i];
+						} catch{}
+					}
+				}
+			}
+			
+			// Next add received invitation users
+			uint[] inviteHandles = this.ReceivedInvitationHandles;
+			if (inviteHandles != null && inviteHandles.Length > 0) {
+				string[] inviteUsers = this.ReceivedInvitations;
+				
+				for (int i = 0; i < inviteUsers.Length; i++) {
+					
+					// update the provider user objects
+					key = ProviderUserManager.CreateKey (inviteUsers[i], protocol);
+							
+					providerUser = null;
+					try {
+						providerUser = ProviderUserManager.GetProviderUser (key);
+						providerUser.TlpConnection = tlpConnection;
+						providerUser.AccountName = this.Name;
+						providerUser.Protocol = this.Protocol;
+						providerUser.ID = inviteHandles[i];
+						providerUser.Relationship = ProviderUserRelationship.ReceivedInvitation;
+					} 
+					catch (Exception fff) {
+						Logger.Debug ("Failed to get ProviderUser {0}", key);
+						Logger.Debug (fff.Message);
+					}
+						
+					if (providerUser == null) {
+						try {
+							providerUser =
+								ProviderUserManager.CreateProviderUser (
+									inviteUsers[i], 
+									this.protocol, 
+									ProviderUserRelationship.ReceivedInvitation);
+							providerUser.AccountName = this.Name;
+							providerUser.TlpConnection = tlpConnection;
+							providerUser.ID = inviteHandles[i];
+						} catch{}
+					}
+				}
+			}
+			
+			// Add subscribed members			
+			string[] members = 
+				tlpConnection.InspectHandles (HandleType.Contact, subscribedGroup.Members);
 			
 			string[] aliasNames = null;
 			if (aliasing == true) {
-				aliasNames = tlpConnection.RequestAliases (cl.Members);
+				aliasNames = tlpConnection.RequestAliases (subscribedGroup.Members);
 				//Logger.Debug ("# returned aliases: {0}", aliasNames.Length);	
 			}	
 
@@ -379,15 +551,15 @@ namespace Banter
 				//Logger.Debug ("MemberID: {0} Member: {1}", cl.Members[i], members[i]);
 				
 				// update the provider user objects
-				string key = ProviderUserManager.CreateKey (members[i], protocol);
-						
-				ProviderUser providerUser = null;
+				key = ProviderUserManager.CreateKey (members[i], protocol);
+				providerUser = null;
 				try {
 					providerUser = ProviderUserManager.GetProviderUser (key);
 					providerUser.TlpConnection = tlpConnection;
 					providerUser.AccountName = this.Name;
 					providerUser.Protocol = this.Protocol;
-					providerUser.ID = cl.Members[i];
+					providerUser.ID = subscribedGroup.Members[i];
+					providerUser.Relationship = ProviderUserRelationship.Linked;
 					if (aliasing == true && aliasNames != null)
 						providerUser.Alias = aliasNames[i];
 				} 
@@ -398,21 +570,25 @@ namespace Banter
 					
 				if (providerUser == null) {
 					try {
-						providerUser = new ProviderUser (tlpConnection, cl.Members[i]);
+						providerUser =
+							ProviderUserManager.CreateProviderUser (
+								members[i], 
+								this.protocol, 
+								ProviderUserRelationship.Linked);
+								
 						providerUser.AccountName = this.Name;
-						providerUser.Protocol = this.Protocol;
-						providerUser.Uri = members[i];
+						providerUser.ID = subscribedGroup.Members[i];
+						providerUser.TlpConnection = tlpConnection;
+					
 						if (aliasing == true && aliasNames != null)
 							providerUser.Alias = aliasNames[i];
-						
-						ProviderUserManager.AddProviderUser (key, providerUser);
 					} catch{}
 				}
 			}
 			
 			ConnectHandlers ();
 			if (presence == true)
-				tlpConnection.RequestPresence (cl.Members);
+				tlpConnection.RequestPresence (subscribedGroup.Members);
 		}
 		
 		private void ConnectHandlers ()
@@ -565,6 +741,60 @@ namespace Banter
 			}
 		}
 		
+
+		/// <summary>
+		/// Callback method when members are added or removed from
+		/// the subscribed list
+		/// </summary>
+		private
+		void
+		OnSubscribedMembersChanged(
+			string message,
+			uint[] added,
+			uint[] removed,
+			uint[] localPending,
+			uint[] remotePending,
+			uint actor,
+			uint reason)
+		{
+			Logger.Debug ("OnSubscribedMembersChanged - called");
+			Logger.Debug ("  Message: {0}", message);
+			Logger.Debug ("  # added: {0}", added.Length);
+			foreach (uint handle in added)
+				Logger.Debug ("    contact id: {0}", handle);
+				
+			Logger.Debug ("  # removed: {0}", removed.Length);
+			foreach (uint handle in removed)
+				Logger.Debug ("    contact id: {0}", handle);
+		}
+		
+
+		/// <summary>
+		/// Callback method when members are added or removed from
+		/// the published list
+		/// </summary>
+		private
+		void
+		OnPublishedMembersChanged(
+			string message,
+			uint[] added,
+			uint[] removed,
+			uint[] localPending,
+			uint[] remotePending,
+			uint actor,
+			uint reason)
+		{
+			Logger.Debug ("OnPublishedMembersChanged - called");
+			Logger.Debug ("  Message: {0}", message);
+			Logger.Debug ("  # added: {0}", added.Length);
+			foreach (uint handle in added)
+				Logger.Debug ("    contact id: {0}", handle);
+				
+			Logger.Debug ("  # removed: {0}", removed.Length);
+			foreach (uint handle in removed)
+				Logger.Debug ("    contact id: {0}", handle);
+		}
+		
 		/// <summary>
 		/// Telepathy callback when new channels are created
 		/// </summary>
@@ -759,6 +989,66 @@ namespace Banter
 			}
 			*/
 		}
+		
+		/// <summary>
+		/// Method to add a member
+		/// </summary>
+		public void AddMember (uint id, string message)
+		{
+			if (this.localInvitationGroup == null)
+				throw new ApplicationException ("Group instance unavailable");
+				
+			uint[] ids = {id};
+			this.localInvitationGroup.AddMembers (ids, message);	
+		}
+
+
+		/// <summary>
+		/// Method to authorize a user who is requesting it to
+		/// enable chat.  
+		/// authorize == false denies the user of authorization
+		/// </summary>
+		public void AuthorizeUser (bool authorize, uint id, string message)
+		{
+			if (this.localInvitationGroup == null)
+				throw new ApplicationException ("Group instance unavailable");
+				
+			uint[] ids = {id};
+			if (authorize == true)
+				localInvitationGroup.AddMembers (ids, message);
+			else
+				localInvitationGroup.RemoveMembers (ids, message);
+		}
+
+		/// <summary>
+		/// Method to invite a user to enable chat
+		/// </summary>
+		public void InviteUser (bool authorize, uint id, string message)
+		{
+			if (this.remoteInvitationGroup == null)
+				throw new ApplicationException ("Group instance unavailable");
+				
+				
+			/*		
+			uint[] ids = {id};
+			if (authorize == true)
+				localInvitationGroup.AddMembers (ids, message);
+			else
+				localInvitationGroup.RemoveMembers (ids, message);
+			*/
+		}
+
+		/// <summary>
+		/// Method to remove an existing user
+		/// </summary>
+		public void RemoveUser (uint id, string message)
+		{
+			if (this.subscribedGroup == null)
+				throw new ApplicationException ("Group instance unavailable");
+				
+			uint[] ids = {id};
+			this.subscribedGroup.RemoveMembers (ids, message);	
+		}
 
 		/// <summary>
 		///	Method to connect this account
@@ -782,6 +1072,8 @@ namespace Banter
 			Logger.Debug ("Account::Disconnect - called");
 			if (connected == true && tlpConnection != null) {
 				try {
+					//localInvitationGroup.Close ();
+					//remoteInvitationGroup.Close ();
 					//DisconnectHandlers ();
 					Logger.Debug ("Calling telepathy disconnect");
 					tlpConnection.Disconnect ();
