@@ -42,7 +42,7 @@ namespace Banter
 		private VBox leftPaneVBox;
 		private VBox rightPaneVBox;
 		private VBox personControlVBox;
-		private Button showPersonDetailsButton;
+		//private Button showPersonDetailsButton;
 		private VBox videoVBox;
 		private VBox audioVBox;
 		private VPaned messagesVPaned;
@@ -57,6 +57,7 @@ namespace Banter
 		private Person peerPerson;
 		private ProviderUser peerProviderUser;
 		private bool hasBeenShown;
+		private bool notifyUser;
 		#endregion
 		
 
@@ -72,19 +73,21 @@ namespace Banter
 			this.chatType = type;
 			conv = ConversationManager.Create(providerUser);
 
+			/*
 			switch(chatType) {
 				case ChatType.Text:
 					conv.AddTextChannel();
 					break;
 				case ChatType.Audio:
 					conv.AddAudioChannel();
-					conv.AddTextChannel();
+					//conv.AddTextChannel();
 					break;
 				case ChatType.Video:
 					conv.AddAudioVideoChannels();
-					conv.AddTextChannel();
+					//conv.AddTextChannel();
 					break;
 			}
+			*/
 
 			peerPerson = person;
 			peerProviderUser = providerUser;
@@ -126,13 +129,15 @@ namespace Banter
 			conv.MessageReceived += OnTextMessageReceived;
 			conv.MessageSent += OnTextMessageSent;
 			conv.MediaChannelClosed += OnMediaChannelClosed;
-			conv.VideoStreamDown += OnVideoStreamDown;
-			conv.AudioStreamDown += OnAudioStreamDown;
 			
 			//conv.VideoStreamUp += OnVideoStreamUp;
 			
 			conv.TextChannelOpened += OnTextChannelOpened;
 			conv.MediaChannelOpened += OnMediaChannelOpened;
+			
+			conv.IncomingAudioCall += OnIncomingAudioCall;
+			conv.IncomingVideoCall += OnIncomingVideoCall;
+			conv.CallHangup += OnCallHangup;
 		}
 
 
@@ -145,13 +150,13 @@ namespace Banter
 			conv.MessageReceived -= OnTextMessageReceived;
 			conv.MessageSent -= OnTextMessageSent;
 			conv.MediaChannelClosed -= OnMediaChannelClosed;
-			conv.VideoStreamDown -= OnVideoStreamDown;
-			conv.AudioStreamDown -= OnAudioStreamDown;
-			
 			//conv.VideoStreamUp += OnVideoStreamUp;
 			
 			conv.TextChannelOpened -= OnTextChannelOpened;
 			conv.MediaChannelOpened -= OnMediaChannelOpened;
+			conv.IncomingAudioCall -= OnIncomingAudioCall;
+			conv.IncomingVideoCall -= OnIncomingVideoCall;
+			conv.CallHangup -= OnCallHangup;
 		}
 		
 		
@@ -164,6 +169,7 @@ namespace Banter
 			hasBeenShown = false;
 			everShown = false;
 			shiftKeyPressed = false;
+			notifyUser = false;
 			
 			// Update the window title
 			Title = string.Format ("Chat with {0}", peerPerson.DisplayName);
@@ -202,13 +208,13 @@ namespace Banter
 			hbox.Show ();
 			personControlVBox.PackStart (hbox, false, false, 0);
 			
-			showPersonDetailsButton = new Button ();
-			showPersonDetailsButton.CanFocus = true;
-			showPersonDetailsButton.Label = Catalog.GetString ("Show Contact _Details");
-			showPersonDetailsButton.UseUnderline = true;
-			showPersonDetailsButton.Image = new Image (Stock.GoBack, IconSize.Menu);
-			showPersonDetailsButton.Show ();
-			hbox.PackStart (showPersonDetailsButton);
+			//showPersonDetailsButton = new Button ();
+			//showPersonDetailsButton.CanFocus = true;
+			//showPersonDetailsButton.Label = Catalog.GetString ("Show Contact _Details");
+			//showPersonDetailsButton.UseUnderline = true;
+			//showPersonDetailsButton.Image = new Image (Stock.GoBack, IconSize.Menu);
+			//showPersonDetailsButton.Show ();
+			//hbox.PackStart (showPersonDetailsButton);
 			
 			videoVBox = new VBox (false, 0);
 			if(this.chatType == ChatType.Video) {
@@ -282,10 +288,10 @@ namespace Banter
 			typingScrolledWindow.Add (typingTextView);
 			
 			Shown += OnWindowShown;
-			DeleteEvent += WindowDeleted;			
+			DeleteEvent += WindowDeleted;
+			this.FocusInEvent += FocusInEventHandler;
 		}
 	
-	
 		///<summary>
 		///	AddMessage
 		/// Addes messages to the messagesView
@@ -298,6 +304,13 @@ namespace Banter
 
 
 		#region EventHandlers
+
+		public void FocusInEventHandler (object o, FocusInEventArgs args)
+		{
+			// remove the Urgency Hint if it was set
+			this.UrgencyHint = false;
+		}
+
 		///<summary>
 		///	OnTextMessageReceived
 		/// Handles all incoming TextMessages and places them into the text chat area
@@ -315,10 +328,17 @@ namespace Banter
 				person = PersonManager.GetPerson (conversation.PeerUser);
 			} catch{}
 			
-			if(person != null)
-				avatarPath = person.GetScaledAvatar(36);
-			
-			AddMessage (message, true, conversation.CurrentMessageSameAsLast, avatarPath);
+			AddMessage (message, true, conversation.CurrentMessageSameAsLast, null);
+
+			// if the window doesn't have focus, notify the user
+			if( (notifyUser) &&(message is TextMessage) ) {
+				if(hasBeenShown && (!HasToplevelFocus)) {
+					this.UrgencyHint = true;
+					NotificationManager.NotifyMessage(person, message);
+				} else {
+					Gnome.Sound.Play(System.IO.Path.Combine(Banter.Defines.SoundDir, "receive.wav"));
+				}
+			}
 		}
 
 		
@@ -395,20 +415,27 @@ namespace Banter
 			ShowAudioControl(false);
 		}
 		
-
+		///<summary>
+		///	OnCallHangup
+		/// Ends the current Internet call
+		///</summary>
+		private void OnCallHangup (Conversation conversation, CallType calltype)
+		{
+			if (calltype == CallType.Audio)
+				ShowAudioControl (false);
+			else if (calltype == CallType.Video)
+				ShowVideoControl (false);
+		}
+		
 		///<summary>
 		///	OnTextMessageSent
 		/// Deals with all TextMessages sent
 		///</summary>
 		private void OnTextMessageSent (Conversation conversation, Message message)
 		{
-			string avatarPath = null;
 			Logger.Debug ("OnMessageSent called: {0}", message.Text);
 
-			if(PersonManager.Me != null)
-				avatarPath = PersonManager.Me.GetScaledAvatar(36);
-				
-			AddMessage (message, false, conversation.CurrentMessageSameAsLast, avatarPath);
+			AddMessage (message, false, conversation.CurrentMessageSameAsLast, null);
 		}
 
 
@@ -489,18 +516,52 @@ namespace Banter
 		private void OnMediaChannelOpened (Conversation conversation)
 		{
 			Logger.Debug("OnMediaChannelOpened was called");
-			
 			Logger.Debug("FIXME: Prompt the idiot to accept right!");
 
+			/*
+			if (conversation.ActiveVideoStream == true ) {
+				// If for some reason we already have a video view, get rid of it and start over
+				if(this.videoView != null) ShowVideoControl(false);
+
+				// Show the Video Control
+				ShowVideoControl(true);
+				conv.StartAudioVideoStreams (videoView.PreviewWindowId, videoView.WindowId);
+			
+			} else if (conversation.ActiveAudioStream == true) {
+				conv.StartAudioStream ();		
+			}
+			*/
+		}
+		
+		///<summary>
+		///	OnIncomingAudioCall
+		/// Called when an incoming audio call is detected
+		///</summary>	
+		private void OnIncomingAudioCall (Conversation conversation)
+		{
+			Logger.Debug ("OnIncomingAudio - called");
+			
+			// Remove then show the audio control
+			ShowAudioControl (false);
+			ShowAudioControl (true);
+			conv.StartAudioStream ();		
+		}
+
+
+		///<summary>
+		///	OnIncomingVideoCall
+		/// Called when an incoming audio call is detected
+		///</summary>	
+		private void OnIncomingVideoCall (Conversation conversation)
+		{
+			Logger.Debug ("OnIncomingVideo - called");
 			// If for some reason we already have a video view, get rid of it and start over
-			if(this.videoView != null)
-				ShowVideoControl(false);
+			if(this.videoView != null) ShowVideoControl(false);
 
 			// Show the Video Control
 			ShowVideoControl(true);
-			conv.StartAudioVideoStreams(videoView.PreviewWindowId, videoView.WindowId);
+			conv.StartAudioVideoStreams (videoView.PreviewWindowId, videoView.WindowId);
 		}
-		
 
 		///<summary>
 		///	OnMediaChannelClosed
@@ -543,7 +604,7 @@ namespace Banter
 
 			ShowAudioControl(false);			
 				
-			if(!conv.ActiveVideoStream)
+			if(!conv.ActiveAudioStream)
 				conv.RemoveMediaChannel();
 		}		
 		
@@ -603,6 +664,7 @@ namespace Banter
 			
 			TextMessage msg = new TextMessage (text, PersonManager.Me.ProviderUser);
 			conv.SendMessage (msg);
+			Gnome.Sound.Play(System.IO.Path.Combine(Banter.Defines.SoundDir, "send.wav"));
 		}
 		
 
@@ -615,8 +677,6 @@ namespace Banter
 			if(hasBeenShown)
 				return;
 			
-			hasBeenShown = true;
-			
 			SetupConversationEvents();
 
 			switch(chatType) {
@@ -625,7 +685,7 @@ namespace Banter
 					// do nothing, text doesn't need to setup streams
 					break;
 				case ChatType.Audio:
-					Logger.Debug("ChatWindow setting up video windows and calling StartAudioVideoStreams");
+					Logger.Debug("ChatWindow setting up video windows and calling StartAudioStream");
 					conv.StartAudioStream();
 					break;
 				case ChatType.Video:
@@ -637,11 +697,15 @@ namespace Banter
 					}
 					break;
 			}		
-		
+
+			notifyUser = false;		
 			// Check for any existing messages including previous chat log
 			Message[] messages = conv.GetReceivedMessages();
 			foreach (Message msg in messages)
 				OnTextMessageReceived (conv, msg);
+			notifyUser = true;
+
+			hasBeenShown = true;
 				
 			// Set the default focus to the TextView where users should type
 			typingTextView.GrabFocus ();
