@@ -37,16 +37,18 @@ namespace Banter
 	public delegate void MediaChannelClosedHandler (Conversation conversation);
 	public delegate void TextChannelOpenedHandler (Conversation conversation);
 	public delegate void TextChannelClosedHandler (Conversation conversation);
-	public delegate void VideoStreamUpHandler (Conversation conversation);
-	public delegate void VideoStreamDownHandler (Conversation conversation);
 	public delegate void VideoStreamPlayingHandler (Conversation conversation);
 	public delegate void VideoStreamStoppedHandler (Conversation conversation);
 	public delegate void VideoStreamErrorHandler (Conversation conversation, string message);
-	public delegate void AudioStreamUpHandler (Conversation conversation);
-	public delegate void AudioStreamDownHandler (Conversation conversation);
 	public delegate void AudioStreamPlayingHandler (Conversation conversation);
 	public delegate void AudioStreamStoppedHandler (Conversation conversation);
 	public delegate void AudioStreamErrorHandler (Conversation conversation, string message);
+	public delegate void IncomingAudioCallHandler (Conversation conversation);
+	public delegate void IncomingVideoCallHandler (Conversation conversation);
+	
+	public delegate void AudioCallIncomingHandler (Conversation conversation);
+	public delegate void CallHangupHandler (Conversation conversation, CallType callType);
+	public delegate void CallRingingHandler (Conversation conversation);
 	
 	public class Conversation : IDisposable
 	{
@@ -56,25 +58,27 @@ namespace Banter
 		public event MediaChannelClosedHandler MediaChannelClosed;
 		public event TextChannelOpenedHandler TextChannelOpened;
 		public event TextChannelClosedHandler TextChannelClosed;
-		public event AudioStreamUpHandler AudioStreamUp;
-		public event AudioStreamDownHandler AudioStreamDown;
 		public event AudioStreamPlayingHandler AudioStreamPlaying;
 		public event AudioStreamStoppedHandler AudioStreamStopped;
 		public event AudioStreamErrorHandler AudioStreamError;
-		public event VideoStreamUpHandler VideoStreamUp;
-		public event VideoStreamDownHandler VideoStreamDown;
 		public event VideoStreamPlayingHandler VideoStreamPlaying;
 		public event VideoStreamStoppedHandler VideoStreamStopped;
 		public event VideoStreamErrorHandler VideoStreamError;
+		public event IncomingVideoCallHandler IncomingAudioCall;
+		public event IncomingVideoCallHandler IncomingVideoCall;
+		public event AudioCallIncomingHandler AudioCallIncoming;
+		public event CallHangupHandler CallHangup;
 		
 		// True the conversation was initiated locally
 		// False the conversation was initiated by an incoming
 		// request from a peer.
 		private bool initiated;
+		private CallType callType;
 		
 		private uint previewWindowID;
 		private uint peerWindowID;
 		private bool outputWindowIsSet = false;
+		private bool inRemoveMedia;
 		private List<Message> messages;
 		private Account account;
 		
@@ -87,12 +91,6 @@ namespace Banter
 		private IChannelStreamedMedia mediaChannel;
 		private IStreamEngine streamEngine;
 		
-		/*
-		private uint textStreamId = 0;
-		private uint audioStreamId = 0;
-		private uint videoStreamId = 0;
-		*/
-		
 		private ProviderUser peerUser;
 		private Presence lastPeerPresence;
 		private uint current;
@@ -186,6 +184,13 @@ namespace Banter
 			txtChannel.Received += OnReceiveMessageHandler;
 			txtChannel.Closed += OnTextChannelClosed;
 			
+			AddPendingMessages ();
+		}
+		#endregion
+		
+		#region Private Methods
+		private void AddPendingMessages ()
+		{
 			// Check for any pending messages and add them to our list
 			try {
 				TextMessage txtMessage;
@@ -197,20 +202,20 @@ namespace Banter
 				}
 			} catch{}
 		}
-		#endregion
 		
-		#region Private Methods
 		private void Init (Account account, ProviderUser providerUser)
 		{
 			this.account = account;
 			this.tlpConnection = this.account.TlpConnection;
 			this.peerUser = providerUser;
+			this.callType = CallType.None;
 			this.messages = new List<Message> ();
 			this.last = 999;
 			this.videoStreams = new Dictionary<uint,uint> ();
 			this.audioStreams = new Dictionary<uint,uint> ();
 			this.peerUser.PresenceUpdated += OnPeerPresenceUpdated;
 			this.lastPeerPresence = this.peerUser.Presence;
+			this.inRemoveMedia = false;  //hack
 		}
 		
 		/// <summary>
@@ -277,18 +282,21 @@ namespace Banter
 				current = this.peerUser.ID;
 			
 				// Indicate the message to registered handlers
-				if (MessageReceived != null){
+				if (MessageReceived != null)
 					MessageReceived (this, txtMessage);
-				}
 			}
 		}
 		
 		private void OnMediaChannelClosed ()
 		{
+			Logger.Debug ("Conversation::OnMediaChannelClosed - called");
+			
+			/*
 			if (MediaChannelClosed != null)
 				MediaChannelClosed (this);
+			*/
 				
-			this.mediaChannel = null;
+			RemoveMediaChannel ();;
 		}
 		
 		private void OnTextChannelClosed()
@@ -323,7 +331,7 @@ namespace Banter
         private void OnStreamAdded (uint streamid, uint contacthandle, org.freedesktop.Telepathy.StreamType streamtype)
 		{
 			Logger.Debug(
-				"OnStreamAdded of type: {0} with id:{1}, for contact {2}", 
+				"Conversation::OnStreamAdded of type: {0} with id:{1}, for contact {2}", 
 				streamtype, 
 				streamid, 
 				contacthandle);
@@ -331,22 +339,20 @@ namespace Banter
 			SaveStream (streamtype, streamid);
 				
 			if (streamtype == StreamType.Video) {
-				if (VideoStreamUp != null)
-					VideoStreamUp (this);
+				callType = CallType.Video;
 			} else if (streamtype == StreamType.Audio) {
-				if (AudioStreamUp != null)
-					AudioStreamUp (this);
+				if (callType != CallType.Video) callType = CallType.Audio;
 			}
 		}
 
 		private void OnStreamDirectionChanged (uint streamid, StreamDirection streamdirection, StreamPendingFlags pendingflags)
         {
-			Logger.Debug("OnStreamDirectionChanged called");
+			Logger.Debug("Conversation::OnStreamDirectionChanged called");
         }
 
         private void OnStreamError (uint streamid, uint errno, string message)
         {
- 			Logger.Debug("OnStreamError called with message: {0}:{1}", errno, message);
+ 			Logger.Debug("Conversation::OnStreamError called with message: {0}:{1}", errno, message);
 
             if (videoStreams.ContainsKey (streamid)) {
             	if (VideoStreamError != null)
@@ -359,19 +365,18 @@ namespace Banter
 
         private void OnStreamRemoved (uint streamid)
         {
+        	Logger.Debug ("Conversation::OnStreamRemoved - called");
+        	
             if (videoStreams.ContainsKey (streamid)) {
                 Logger.Debug ("Removing video stream: {0}", streamid);
                 videoStreams.Remove (streamid);
-                
-            	if (VideoStreamDown != null)
-            		VideoStreamDown (this);
             } else if (audioStreams.ContainsKey (streamid)) {
                 Logger.Debug ("Removing audio stream: {0}", streamid);
                 audioStreams.Remove (streamid);
-                
-            	if (AudioStreamDown != null)
-            		AudioStreamDown (this);
             }
+            
+            if (audioStreams.Count == 0 && videoStreams.Count == 0)
+            	this.RemoveMediaChannel ();
         }
 
         private void OnStreamStateChanged (uint streamid, org.freedesktop.Telepathy.StreamState streamstate)
@@ -400,11 +405,7 @@ namespace Banter
 	       		{
 	       			//IndicateSystemMessage ("Video chat connected!");
 					if (videoStreams.ContainsKey(streamid)) {
-						if (VideoStreamUp != null) 
-							VideoStreamUp (this);
 	       			} else if (audioStreams.ContainsKey (streamid)) {
-						if (AudioStreamUp != null) 
-							AudioStreamUp (this);
 	       			}
 	       			break;
 	       		}
@@ -414,7 +415,6 @@ namespace Banter
 					if (videoStreams.ContainsKey(streamid)) {
 						if (VideoStreamPlaying != null)
 							VideoStreamPlaying (this);
-						//IndicateSystemMessage ("Video chat started");	
 	       			} else if (audioStreams.ContainsKey (streamid)) {
 						if (AudioStreamPlaying != null) 
 							AudioStreamPlaying (this);
@@ -427,7 +427,6 @@ namespace Banter
 					if (videoStreams.ContainsKey(streamid)) {
 						if (VideoStreamStopped != null) 
 							VideoStreamStopped (this);
-						IndicateSystemMessage ("Video chat stopped");	
 	       			} else if (audioStreams.ContainsKey (streamid)) {
 						if (AudioStreamStopped != null) 
 							AudioStreamStopped (this);
@@ -487,7 +486,6 @@ namespace Banter
 			mediaChannel.StreamStateChanged += OnStreamStateChanged;
 			mediaChannel.MembersChanged += OnMembersChanged;
         }
-		
 		#endregion
 		
 		#region Internal Methods
@@ -507,6 +505,30 @@ namespace Banter
 				MediaChannelOpened (this);
 		}
 		
+		internal void IndicateAudioCall ()
+		{
+			if (IncomingAudioCall != null) IncomingAudioCall (this);
+		}
+		
+		internal void IndicateVideoCall ()
+		{
+			if (IncomingVideoCall != null) IncomingVideoCall (this);
+		}
+		
+		internal void IndicateReceivedMessages ()
+		{
+			if (this.MessageReceived != null) {
+				AddPendingMessages ();
+				Banter.Message[] messages =
+					this.GetReceivedMessages ();
+				if (messages.Length > 0) {
+					foreach (Message msg in messages)
+						this.MessageReceived (this, msg);
+				}
+			} else {
+				Logger.Debug ("No registered receive handler");
+			}
+		}
 		#endregion
 		
 		#region Public Methods
@@ -514,9 +536,13 @@ namespace Banter
 		{
 			if (txtChannel != null)	{
 				try {txtChannel.Close();} catch{}
-				//txtChannel.Close();
 				txtChannel = null;
 			}
+			
+			if (this.mediaChannel != null) {
+				try {this.mediaChannel.Close();} catch{}
+				mediaChannel = null;
+			}
 		}
 		
 		public Banter.Message[] GetReceivedMessages()
@@ -524,6 +550,7 @@ namespace Banter
 			return messages.ToArray();
 		}
 		
+		
 		/// <summary>
 		/// Method to indicate a local system message
 		/// </summary>
@@ -542,18 +569,19 @@ namespace Banter
 		{
 			// FIXME::Throw exception
 			if (tlpConnection == null) return;
-			if (txtChannel == null) return;
 			if (message == null) return;
 			
-			this.txtChannel.Send (org.freedesktop.Telepathy.MessageType.Normal, message.Text);
-
-			if (current != 0)
-				last = current;
-				
-			current = tlpConnection.SelfHandle;
-			
-			if (MessageSent != null)
-				MessageSent (this, message);
+			try {
+				// If a text channel doesn't exist attempt to create one
+				if (txtChannel == null)	AddTextChannel ();
+				this.txtChannel.Send (org.freedesktop.Telepathy.MessageType.Normal, message.Text);
+				if (current != 0) last = current;
+				current = tlpConnection.SelfHandle;
+				if (MessageSent != null) MessageSent (this, message);
+			} catch (Exception sm) {
+				Logger.Debug ("Conversation::SendMessage failed");
+				Logger.Debug (sm.Message);
+			}
 		}
 		
 		/// New methods 6/7
@@ -578,6 +606,17 @@ namespace Banter
 			txtChannel.Closed += OnTextChannelClosed;
 		}
 
+		/// <summary>
+		/// Method to open and setup a text channel
+		/// </summary>
+		public void AddTextChannel (IChannelText existingTxtChannel)
+		{
+			if (txtChannel != null) return;
+			txtChannel = existingTxtChannel;				
+			txtChannel.Received += OnReceiveMessageHandler;
+			txtChannel.Closed += OnTextChannelClosed;
+		}
+
 		/// <summary>
 		/// Method to open and connect and audio
 		/// channel with a peer
@@ -616,33 +655,33 @@ namespace Banter
 		/// </summary>
 		public void RemoveMediaChannel ()
 		{
+			Logger.Debug ("RemoveMediaChannel - called");
 			if (mediaChannel == null) return;
 			outputWindowIsSet = false;
 
-			
+			if (inRemoveMedia == false ) {
+		       	if (callType == CallType.Audio)
+		       		IndicateSystemMessage ("Audio call ending");
+		       	else if (callType == CallType.Video)
+		       		IndicateSystemMessage ("Video call ending");
+
+		       	if (CallHangup != null) CallHangup (this, callType);
+	       	}
+
+			inRemoveMedia = true;			
 			uint i = 0;
 			try {
-				uint[] streams = new uint[videoStreams.Count + audioStreams.Count];
-				
-      			foreach (KeyValuePair<uint,uint> kvp in videoStreams)
-      				streams[i++] = kvp.Value;
-      				
-      			foreach (KeyValuePair<uint,uint> kvp in audioStreams)
-      				streams[i++] = kvp.Value;
-					
-				((IChannelStreamedMedia) mediaChannel).RemoveStreams (streams);					
-	
 				audioStreams.Clear();
 				videoStreams.Clear();
 				mediaChannel.Close ();
 				Logger.Debug ("Completed Media Channel cleanup");
 			} catch{}
 			
-			IndicateSystemMessage ("Video chat stopped");	
 			mediaChannel = null;
+			callType = CallType.None;
+			inRemoveMedia = false;
 		}
 		
-		
 		/// <summary>
 		/// Method to start streaming the audio channel
 		/// </summary>
@@ -650,8 +689,9 @@ namespace Banter
 		{
 			Logger.Debug ("StartAudioStream - called");
 			
+			if (mediaChannel == null) SetupMediaChannel ();
 			if (mediaChannel == null)
-				throw new ApplicationException ("Streamed media channel does not exist");
+				throw new ApplicationException ("Failed to get a media channel");
 				
 			IChannelHandler	channelHandler = 
 				Bus.Session.GetObject<IChannelHandler> (
@@ -695,6 +735,8 @@ namespace Banter
 				uint[] handles = new uint [] {peerUser.ID};
 				StreamInfo[] infos = mediaChannel.RequestStreams (handles[0], streamtypes);
 			}
+			
+			IndicateSystemMessage ("Audio call started");
 		}
 		
 		/// <summary>
@@ -705,8 +747,9 @@ namespace Banter
 		{
 			Logger.Debug ("StartAudioVideoStreams - called");
 			
+			if (mediaChannel == null) SetupMediaChannel ();
 			if (mediaChannel == null)
-				throw new ApplicationException ("Media stream channel does not exist");
+				throw new ApplicationException ("Failed to get a media channel");
 				
 			this.peerWindowID = peerwindowId;
 			this.previewWindowID = previewWindowId;
@@ -737,43 +780,31 @@ namespace Banter
                            info.Id, info.Type, info.ContactHandle, info.Direction);
 
 				// Save the type of the stream so we can reference it later
-                SaveStream (info.Type, info.Id);	       	
-
-				/*
-	       		Logger.Debug ("Setting peer Window ID - Object Path: {0}", this.videoChannelObjectPath.ToString());
-	       		Logger.Debug ("info ID: {0}  Contact Handle: {1}", info.Id, info.ContactHandle);
-	       		tempStreamId = info.Id;
-	       	
-	       		Logger.Debug(
-	       			"Stream Info: Id:{0}, Type:{1}, ContactHandle:{2}, Direction: {3}",
-	       			info.Id,
-	       			info.Type,
-	       			info.ContactHandle,
-	       			info.Direction);
-	       				
-	       		// Save the type of the stream so we can reference it later
-	       		//SaveStream (info.Type, info.Id);
-		       */
+                SaveStream (info.Type, info.Id);
 			}
 		       
-			Logger.Debug("Getting the stream_engine");
-				
 			streamEngine = 
 				Bus.Session.GetObject<IStreamEngine> (
 					"org.freedesktop.Telepathy.StreamEngine",
 	           		new ObjectPath ("/org/freedesktop/Telepathy/StreamEngine"));
 
-	        Logger.Debug("have the stream engine");
-		        
-			Logger.Debug("Adding Preview Window {0}", previewWindowId);
-		    streamEngine.AddPreviewWindow(previewWindowId);
+			Logger.Debug ("Adding Preview Window {0}", previewWindowId);
+		    streamEngine.AddPreviewWindow (previewWindowId);
+			IndicateSystemMessage ("Video call started");
+
+			/*
+			if (this.videoStreams.Count > 0) {
+				Logger.Debug("Adding Preview Window {0}", previewWindowId);
+			    streamEngine.AddPreviewWindow(previewWindowId);
+				IndicateSystemMessage ("Video chat started");
+			} else
+				IndicateSystemMessage ("Audio chat started")
+			*/
+
 			streamEngine.Receiving += OnStreamEngineReceiving;
-
-			IndicateSystemMessage ("Video chat started");
-
+			
 			if (this.initiated == true) {
 				uint[] streamtypes = new uint[2];
-					
 				streamtypes[0] = (uint) StreamType.Audio;
 				streamtypes[1] = (uint) StreamType.Video;
 
