@@ -84,8 +84,11 @@ namespace Banter
 		private VBox groupButtonsVBox;
 		private Button newGroupButton;
 		
-		private Button settingsButton;
-		private Button addPersonButton;
+		private SidebarTextButton settingsButton;
+		private SidebarTextButton addPersonButton;
+		private SidebarTextButton removePersonButton;
+		private bool removeEnabled;
+		private Tooltips dialogTips;
 		
 		//
 		// PersonView Pane
@@ -120,6 +123,7 @@ namespace Banter
 				"0" // show everyone
 			);
 			saveNeeded = true;
+			removeEnabled = false;
 		}
 		
 		public GroupWindow (string guidStr) : base (Catalog.GetString ("Banter"))
@@ -176,7 +180,7 @@ namespace Banter
 		}
 		#endregion
 
-		#region public methods
+		#region Public methods
 		// <summary>
 		// Save the state of the group window (i.e., position, size, selected group,
 		// person card size, etc.
@@ -206,7 +210,7 @@ namespace Banter
 			personCardSize = personView.PersonCardSize;
 			personGroup = selectedGroup;
 			
-			Logger.Debug (
+/*			Logger.Debug (
 					"Window State:\n" +
 					"\tVisible: {0}\n" +
 					"\tGUID: {1}\n" +
@@ -223,11 +227,11 @@ namespace Banter
 					personCardSize,
 					personGroup == null ?
 						"Everyone" : personGroup.DisplayName);
-			
+*/			
 			string keyPrefix = Preferences.GroupWindowPrefix + "/" + guidStr;
 			string key;
 			
-			Logger.Info ("Saving window state: '{0}'...", Title);
+//			Logger.Info ("Saving window state: '{0}'...", Title);
 			try {
 				key = keyPrefix + ConfStateVisible;
 				Preferences.Set (key, visible);
@@ -338,11 +342,11 @@ namespace Banter
 		{
 			this.guid = guid;
 			searchTimeoutID = 0;
-			
+
 			CreateWidgets ();
 			
 			SetDefaultSize (width, height);
-			Icon = Utilities.GetIcon ("banter", 16);
+			Icon = Utilities.GetIcon ("banter-22", 22);
 			AllowShrink = true;
 			
 			initialPersonCardSize = personCardSize;
@@ -362,6 +366,8 @@ namespace Banter
 		
 		private void CreateWidgets ()
 		{
+			dialogTips = new Tooltips();			
+
 			VBox vbox = new VBox (false, 0);
 			
 			// Menubar?  TODO
@@ -407,7 +413,7 @@ namespace Banter
 			l.UseMarkup = true;
 			l.Show ();
 			vbox.PackStart (l, false, false, 0);
-			
+
 			// Search Entry
 //			vbox.PackStart (CreateSidebarSearchEntry (), false, false, 0);
 			
@@ -432,6 +438,11 @@ namespace Banter
 			avatarSelector.Show();
 
 			hbox.PackStart (avatarSelector, false, false, 0);
+			dialogTips.SetTip (avatarSelector,
+                        Catalog.GetString("Change your avatar"),
+                        Catalog.GetString("Click here to change your avatar"));
+
+
 			
 			VBox vbox = new VBox (false, 0);
 			vbox.Show ();
@@ -448,6 +459,9 @@ namespace Banter
 			statusEntry.PresenceChanged += OnStatusEntryChanged;
 			statusEntry.Show ();
 			vbox.PackStart (statusEntry, false, false, 0);
+			dialogTips.SetTip (statusEntry,
+                        Catalog.GetString("Change your online status"),
+                        Catalog.GetString("Change your online status"));
 			
 //			statusComboBoxEntry = ComboBoxEntry.NewText ();
 //			statusComboBoxEntry.KeyPressEvent += OnStatusComboKeyPress;
@@ -462,7 +476,7 @@ namespace Banter
 		
 		private void OnStatusEntryChanged (Presence presence)
 		{
-			Logger.Debug ("Setting presence on Me to {0}", presence.Message);
+			//Logger.Debug ("Setting presence on Me to {0}", presence.Message);
 			if (PersonManager.Me != null) {
 				PersonManager.Me.SetStatus (presence);
 				
@@ -547,21 +561,24 @@ namespace Banter
 			VBox buttonsVBox = new VBox (false, 0);
 
 			// Everyone
-			everyoneGroupButton = CreateSidebarTextButton (
-					Catalog.GetString ("Everyone"));
+			everyoneGroupButton = new SidebarTextButton(Catalog.GetString ("Everyone"));
+			everyoneGroupButton.Show();
 			everyoneGroupButton.Clicked += OnEveryoneClicked;
 			buttonsVBox.PackStart (everyoneGroupButton, false, false, 0);
-			
+
 			// Placeholder for other groups
 			groupButtonsVBox = new VBox (false, 0);
 			vbox.Show ();
 			buttonsVBox.PackStart (groupButtonsVBox, false, false, 0);
 			
 			// New Group
+			// Add this back in when we have groups figured out
+			/*
 			newGroupButton = CreateSidebarTextButton (
 					Catalog.GetString ("New Group..."));
 			newGroupButton.Clicked += OnNewGroupClicked;
 			buttonsVBox.PackStart (newGroupButton, false, false, 0);
+			*/
 			
 			buttonsVBox.Show ();
 			spacerHBox.PackStart (buttonsVBox, true, true, 0);
@@ -591,16 +608,24 @@ namespace Banter
 			VBox buttonsVBox = new VBox (false, 0);
 
 			// Settings
-			settingsButton = CreateSidebarTextButton (
-					Catalog.GetString ("Settings"));
+			settingsButton = new SidebarTextButton(Catalog.GetString ("Settings"));
+			settingsButton.Show();
 			settingsButton.Clicked += OnSettingsClicked;
 			buttonsVBox.PackStart (settingsButton, false, false, 0);
 			
 			// Add Person
-			addPersonButton = CreateSidebarTextButton (
+			addPersonButton = new SidebarTextButton (
 					Catalog.GetString ("Add Person..."));
+			addPersonButton.Show();
 			addPersonButton.Clicked += OnAddPersonClicked;
 			buttonsVBox.PackStart (addPersonButton, false, false, 0);
+
+			// Remove Person
+			removePersonButton = new SidebarTextButton (
+					Catalog.GetString ("Remove Person"));
+			removePersonButton.Show();
+			removePersonButton.Clicked += OnRemovePersonClicked;
+			buttonsVBox.PackStart (removePersonButton, false, false, 0);
 			
 			buttonsVBox.Show ();
 			spacerHBox.PackStart (buttonsVBox, true, true, 0);
@@ -639,26 +664,6 @@ namespace Banter
 			buttonBox.Show ();
 			return buttonBox;
 		}
-		
-		private Button CreateSidebarTextButton (string buttonText)
-		{
-			Button button;
-			HBox hbox;
-			Label buttonLabel;
-			
-			hbox = new HBox (false, 0);
-			buttonLabel = new Label (buttonText);
-			buttonLabel.Xalign = 0;
-			buttonLabel.UseUnderline = false;
-			buttonLabel.UseMarkup = true;
-			buttonLabel.Show ();
-			hbox.PackStart (buttonLabel, false, false, 4);
-			button = new Button (hbox);
-			button.Relief = ReliefStyle.None;
-			button.Show ();
-			
-			return button;
-		}
 
 		private bool SearchCallback ()
 		{
@@ -741,9 +746,35 @@ Logger.Debug ("GroupWindow.BuildGroupButtonsView adding {0} groups",
 
 			return base.OnDeleteEvent (evnt);
 		}
+
+
+
 		#endregion
 		
-		#region event handlers
+		#region Event handlers
+
+		private void PersonMeArrivedHandler (Person person)
+		{
+			// Set "my" display name
+			Person me = PersonManager.Me;
+			if (me != null) {
+				myDisplayName.Markup =
+					string.Format (
+						"<span weight=\"bold\" size=\"small\">{0}</span>",
+						me.DisplayName);
+				
+				// Set "my" status
+				statusEntry.Presence = me.Presence;
+				
+				Logger.Debug ("FIXME: Populate the StatusEntry widget with saved status messages.");
+
+				if(me.Photo != null)
+					avatarSelector.Pixbuf = me.Photo;
+				
+				me.PresenceUpdated += OnMyPresenceUpdated;
+			}
+		}
+
 		private void OnRealizeWidget (object sender, EventArgs args)
 		{
 			// Set "my" display name
@@ -759,7 +790,12 @@ Logger.Debug ("GroupWindow.BuildGroupButtonsView adding {0} groups",
 				
 				Logger.Debug ("FIXME: Populate the StatusEntry widget with saved status messages.");
 				
+				if(me.Photo != null)
+					avatarSelector.Pixbuf = me.Photo;
+
 				me.PresenceUpdated += OnMyPresenceUpdated;
+			} else {
+				PersonManager.Instance.PersonMeArrived += PersonMeArrivedHandler;
 			}
 			
 			// Fill out the existing groups
@@ -795,6 +831,8 @@ Logger.Debug ("GroupWindow.BuildGroupButtonsView adding {0} groups",
 			// Cause brand new windows to be saved.  Previously existing windows
 			// will be ignored unless the user moves them.
 			SaveState (); 
+			
+			//Gnome.Sound.Play(System.IO.Path.Combine(Banter.Defines.SoundDir, "banter.wav"));				
 		}
 		
 		private void OnSearchEntryChanged (object sender, EventArgs args)
@@ -839,7 +877,8 @@ Logger.Debug ("GroupWindow.BuildGroupButtonsView adding {0} groups",
 				new HIGMessageDialog (this, Gtk.DialogFlags.DestroyWithParent,
 					Gtk.MessageType.Question,
 					Gtk.ButtonsType.OkCancel,
-					Catalog.GetString ("New group window"),
+					Catalog.GetString ("Banter - New group window"),
+					Catalog.GetString ("Add new group"),
 					Catalog.GetString ("Enter the name of the new group you'd like to create."));
 
 			Gtk.Entry groupNameEntry = new Entry ();
@@ -868,10 +907,40 @@ Logger.Debug ("GroupWindow.BuildGroupButtonsView adding {0} groups",
 		{
 			Application.ActionManager ["ShowPreferencesAction"].Activate ();
 		}
+
+		private void OnRemovePersonClicked (object sender, EventArgs args)
+		{
+			personView.EnableRemoveButtons(!removeEnabled);
+			removeEnabled = !removeEnabled;
+			if(removeEnabled)
+				removePersonButton.Label = Catalog.GetString ("Done Removing");
+			else
+				removePersonButton.Label = Catalog.GetString ("Remove People");
+		}
 		
 		private void OnAddPersonClicked (object sender, EventArgs args)
 		{
-			Logger.Debug ("FIXME: Implement GroupWindow.OnAddPersonClicked ()");
+			HIGMessageDialog dialog =
+				new HIGMessageDialog (this, Gtk.DialogFlags.DestroyWithParent,
+					Gtk.MessageType.Question,
+					Gtk.ButtonsType.OkCancel,
+					Catalog.GetString ("Banter - Add person"),
+					Catalog.GetString ("Invite a new person"),
+					Catalog.GetString ("Enter the email address of the person you'd like to add."));
+
+			Gtk.Entry emailEntry = new Entry ();
+            emailEntry.ActivatesDefault = true;
+			dialog.ExtraWidget = emailEntry;
+			
+			int returnCode = dialog.Run ();
+			
+			if (returnCode == (int) Gtk.ResponseType.Ok) {
+				string emailAddress = emailEntry.Text.Trim ();
+				// do stuff to add the new person
+				Logger.Debug("FIXME: Add code to invite the person {0}", emailAddress);
+			}
+			
+			dialog.Destroy ();
 		}
 		
 		private void OnToggleSidebarButtonClicked (object sender, EventArgs args)
@@ -1010,6 +1079,7 @@ Logger.Debug ("GroupWindow.BuildGroupButtonsView adding {0} groups",
 			HIGMessageDialog dialog = new HIGMessageDialog (
 				this, DialogFlags.DestroyWithParent,
 				MessageType.Warning, ButtonsType.YesNo,
+				Catalog.GetString("Banter - Delete Group"),
 				string.Format (
 					Catalog.GetString ("Delete \"{0}\"?"),
 					group.DisplayName),
