@@ -25,7 +25,7 @@ using System.Collections.Generic;
 using System.IO;
 using System.Net;
 using System.Text;
-using Evolution;
+//using Evolution;
 
 namespace Banter
 {
@@ -41,11 +41,12 @@ namespace Banter
 	public class Person
 	{
 		#region Private Types
-		private Evolution.Contact edsContact;
+		//private Evolution.Contact edsContact;
 		private string cachePath;
 		private Gdk.Pixbuf avatar;
 		private List<ProviderUser> providerUsers;
 		private Presence presence;
+		private string displayName;
 		#endregion
 
 		#region Public Events
@@ -78,19 +79,7 @@ namespace Banter
 		/// </summary>
 		public string DisplayName
 		{
-			get
-			{
-				string displayName = String.Empty;
-				
-				if (edsContact != null) {
-					if ((edsContact.FileAs != null) && (edsContact.FileAs.Length > 0) ) {
-						displayName = edsContact.FileAs;
-						return displayName;
-					}
-				}
-				
-				return displayName;
-			}
+			get { return displayName; }
 		}
 
 
@@ -102,18 +91,25 @@ namespace Banter
 			get 
 			{		
 				string jabberId = String.Empty;
+
+
+				if(providerUsers.Count > 0) {
+					jabberId = providerUsers[0].Uri;
+				}
 				
+				/*
 				if( (edsContact.ImJabber != null) && (edsContact.ImJabber.Length > 0) )
 					jabberId = edsContact.ImJabber[0];
-					
+				*/	
 				return jabberId;
 			}
-			set
+/*			set
 			{
 				string[] jabberId = new string[1];
 				jabberId[0] = value;
 				edsContact.ImJabber = jabberId;
 			}
+*/
 		}
 
 
@@ -157,18 +153,20 @@ namespace Banter
 		/// <summary>
 		/// The internal Evolution contact
 		/// </summary>
+		/*
 		public Evolution.Contact EDSContact
 		{
 			get{ return edsContact; }
 			set{ edsContact = value; }
 		}
-		
+		*/
+
 		/// <summary>
 		/// The Id of this Person
 		/// </summary>
 		public string Id
 		{
-			get{ return edsContact.Id;}
+			get{ return JabberId;}
 		}		
 
 
@@ -203,64 +201,40 @@ namespace Banter
 		/// <summary>
 		/// Constructs a person from a tapioca contact
 		/// </summary>
-		internal Person(Evolution.Contact edsContact)
+		/*internal Person(Evolution.Contact edsContact)
 		{
 			this.edsContact = edsContact;
 			Init();
 		}
-
+		*/
 		
 		/// <summary>
 		/// Constructs a person with a displayName
 		/// </summary>
-		public Person(string displayName)
+		public Person(ProviderUser user)
 		{
-			this.edsContact = new Contact();
-			edsContact.FileAs = displayName;
-			Init();
+			providerUsers = new List<ProviderUser> ();
+
+			if(user != null) {
+				providerUsers.Add(user);
+				if(user.Presence != null)
+					presence = user.Presence;
+				user.PresenceUpdated += ProviderUserPresenceUpdated;
+				user.AvatarTokenUpdated += this.ProviderUserAvatarTokenUpdated;
+				user.AvatarReceived += this.ProviderUserAvatarReceived;
+				if(user.Alias.Length > 0)
+					displayName = user.Alias;
+				else
+					displayName = user.Uri;
+			} 
+
+			if(presence == null)
+				presence = new Presence(PresenceType.Offline);
 		}
 		#endregion
 		
 
 		#region Private Methods		
-		private void Init()
-		{
-		
-			// init internal types
-			providerUsers = new List<ProviderUser> ();
-			presence = new Presence(PresenceType.Offline);
-
-			// first check to see if this is a real edsContact
-			if(edsContact.Id != null) {
-				string homeDirectoryPath = Environment.GetFolderPath (Environment.SpecialFolder.Personal);
-				cachePath = System.IO.Path.Combine (homeDirectoryPath, ".banter/Cache");
-				cachePath = System.IO.Path.Combine (cachePath, this.Id);
-
-				try
-				{
-					//Logger.Debug("FIXME: A Person's Cache needs to be cleaned up somewhere too");
-					if(!System.IO.Directory.Exists(cachePath)) {
-						System.IO.Directory.CreateDirectory(cachePath);
-					}
-				}
-				catch(Exception e)
-				{
-					Logger.Debug(e.Message);
-				}
-				
-				// Read the contact's Avatar from EDS if it's there
-				ContactPhoto photo = edsContact.Photo;
-				if(photo.PhotoType == ContactPhotoType.Inlined) {  // the photo is stored in EDS
-					avatar = new Gdk.Pixbuf(photo.Data);
-				}
-				else if(photo.PhotoType == ContactPhotoType.Uri) {				
-					// not sure how to handle a Uri based photo
-					Logger.Debug("FIXME: handle Uri based photo");
-				}
-			}
-		}
-
-		
 		/// <summary>
 		/// Updates the persons presence to be the most present of his ProviderUsers
 		/// </summary>
@@ -269,21 +243,26 @@ namespace Banter
 			//Logger.Debug("FIXME: Person.UpdatePresence should use a policy to get the right presence");
 			if(providerUsers.Count > 0) {
 				presence = providerUsers[0].Presence;
+
+				// check the display name while we are here
+				if( (providerUsers[0].Alias != null) && (providerUsers[0].Alias.Length > 0) ) {
+					displayName = providerUsers[0].Alias;
+ 				}
+
+				// Call the event on the GUI thread		
+				if(PresenceUpdated != null)
+				{
+					Gtk.Application.Invoke (delegate {
+						PresenceUpdated(this);
+					});			
+				}
 			}
 
-			// Call the event on the GUI thread				
-			if(PresenceUpdated != null)
-			{
-				Gtk.Application.Invoke (delegate {
-					PresenceUpdated(this);
-				});			
-			}
 		}
-		
-		
+
 		private void ProviderUserPresenceUpdated (ProviderUser user)
 		{
-			//Logger.Debug("Presence updated for ProviderUser: {0}", user.Alias);			
+			Logger.Debug("Person:ProviderUserPresenceUpdated for ProviderUser: {0}", user.Alias);			
 			UpdatePresence();
 		}
 		
@@ -325,92 +304,6 @@ namespace Banter
 		
 		
 		#region Public Methods
-		/// <summary>
-		/// Return the path to the photo size requested
-		/// </summary>
-		public string GetScaledAvatar(int size)
-		{
-			if(this.Photo == null)
-				return null;
-				
-			string	avatarPath = null;
-			bool	scalePhoto = false;
-			
-			switch(size) {
-				case 16:
-					avatarPath = Path.Combine(cachePath, "16x16");
-					scalePhoto = true;
-					break;
-				case 24:
-					avatarPath = Path.Combine(cachePath, "24x24");
-					scalePhoto = true;
-					break;
-				case 36:
-					avatarPath = Path.Combine(cachePath, "36x36");
-					scalePhoto = true;
-					break;
-				case 48:
-					avatarPath = Path.Combine(cachePath, "48x48");
-					scalePhoto = true;
-					break;
-				default:
-					avatarPath = Path.Combine(cachePath, "actual");
-					break;
-			}
-
-			try
-			{
-				if(!System.IO.File.Exists(avatarPath)) {
-					byte[] data;
-					
-					System.IO.FileStream fStream = new FileStream(avatarPath, FileMode.Create);
-					if(scalePhoto) {
-						Gdk.Pixbuf scaled = avatar.ScaleSimple(size, size, Gdk.InterpType.Bilinear);
-						data = scaled.SaveToBuffer("png");
-					}
-					else {
-						data = avatar.SaveToBuffer("png");
-					}
-					
-					fStream.Write(data, 0, data.Length);
-					fStream.Close();
-				}
-			}
-			catch(Exception e)
-			{
-				Logger.Debug(e.Message);
-			}
-
-			return avatarPath;
-		}
-
-
-		/// <summary>
-		/// Ensures that we have ProviderUsers for eds Provider values
-		/// </summary>
-		public void UpdateProviderUsers()
-		{
-			//Logger.Debug("FIXME: Person.UpdateProviderUsers should use a policy for the order");
-			providerUsers.Clear();
-		
-			// Jabber values
-			foreach(string uri in edsContact.ImJabber) {
-				string key = ProviderUserManager.CreateKey(uri, ProtocolName.Jabber);
-				ProviderUser providerUser = ProviderUserManager.GetProviderUser(key);
-				if(providerUser == null) {
-					providerUser = ProviderUserManager.CreateProviderUser(uri, ProtocolName.Jabber);
-				}
-				
-				if(providerUser != null) {
-					providerUsers.Add(providerUser);
-					providerUser.PresenceUpdated += ProviderUserPresenceUpdated;
-					providerUser.AvatarTokenUpdated += this.ProviderUserAvatarTokenUpdated;
-					providerUser.AvatarReceived += this.ProviderUserAvatarReceived;
-				}
-			}
-			UpdatePresence();
-		}
-		
 		/// <summary>
 		/// Sets the status for the person if the person IsMe
 		/// </summary>
