@@ -28,6 +28,7 @@ namespace Banter
 {
 
 	public delegate void ProviderUserAddedHandler (ProviderUser user);
+	public delegate void ProviderUserRemovedHandler (string uri);
 	
 	///<summary>
 	///	ProviderUserManager Class
@@ -48,6 +49,7 @@ namespace Banter
 
 		#region Public Events
 		public static event ProviderUserAddedHandler ProviderUserAdded;
+		public static event ProviderUserRemovedHandler ProviderUserRemoved;
 		#endregion
 		
 
@@ -183,6 +185,49 @@ namespace Banter
 		
 
 
+		/// <summary>
+		/// Creates a new ProviderUser and adds them in one atomic operation
+		/// </summary>	
+		public static ProviderUser CreateProviderUser(string uri, string protocol, ProviderUserRelationship relationship)
+		{
+			lock(locker) {
+				string key = CreateKey (uri, protocol);
+				
+				if (!ProviderUserManager.Instance.users.ContainsKey (key)) {
+					ProviderUser user = new ProviderUser();
+					user.Uri = uri;
+					user.Protocol = protocol;
+					user.Relationship = relationship;
+					ProviderUserManager.Instance.users[key] = user;
+					if (ProviderUserAdded != null)
+						ProviderUserAdded (user);					
+					return user;
+				}
+				else
+					throw new ApplicationException("key already exists");
+			}
+		}		
+		
+
+
+		/// <summary>
+		/// Method to remove a provider user from the list
+		/// </summary>	
+		public static void RemoveProviderUser(string uri, string protocol)
+		{
+			lock (locker) {
+				string key = CreateKey (uri, protocol);
+				
+				if (ProviderUserManager.Instance.users.ContainsKey (key) == true)
+				{
+					ProviderUserManager.Instance.users.Remove (key);
+					if(ProviderUserRemoved != null) {
+						ProviderUserRemoved(uri);
+					}
+				}
+			}
+		}		
+		
 		/// <summary>
 		/// Generates a key to locate a ProviderUser
 		/// </summary>	
